/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach.bayernid.proxy.message;

import java.io.IOException;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;

import de.akdb.egov.bsp.nachrichten.AbsenderType;
import de.akdb.egov.bsp.nachrichten.BspNachricht;
import de.akdb.egov.bsp.nachrichten.BspQuittung;
import de.akdb.egov.bsp.nachrichten.DataContainerType;
import de.akdb.egov.bsp.nachrichten.SchluesseltabelleType;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.GrpcAbsender;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.GrpcBayernIdMessageMetadata;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.GrpcSendBayernIdMessageResponse;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.message.BayernIdMessage.BayernIdMessageBuilder;

@Mapper(nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE, nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface BayernIdMessageMapper {

	String TABELLE_NUMMER_TEXT_ENCODING = "9004";
	String TABELLE_NUMMER_MIME_TYPE = "9005";
	String TABELLE_NUMMER_GEMEINDE_SCHLUESSEL = "36";
	String TEXT_ENCODING_PLAIN_TEXT = "text/plain";

	@Mapping(target = "attachments", ignore = true)
	BayernIdMessageBuilder toBayernIdMessage(GrpcBayernIdMessageMetadata grpcMessage);

	@Mapping(target = "gemeindeschluessel.tabelle", constant = TABELLE_NUMMER_GEMEINDE_SCHLUESSEL)
	@Mapping(target = "gemeindeschluessel.schluessel", source = "gemeindeschluessel")
	AbsenderType toAbsenderType(GrpcAbsender absender);

	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "statusBytes", ignore = true)
	@Mapping(target = "messageBytes", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	GrpcSendBayernIdMessageResponse fromBayernIdResponse(BayernIdResponse message);

	@Mapping(target = "nachrichtenKopf.identifikationNachricht.nachrichtenId", source = "messageId")
	@Mapping(target = "nachrichtenKopf.identifikationNachricht.erstellungszeitpunkt", source = "createdAt")
	@Mapping(target = "nachrichtenKopf.empfaenger.postkorbId", source = "empfaenger.postkorbId")
	@Mapping(target = "nachrichtenKopf.empfaenger.name", source = "empfaenger.name")
	@Mapping(target = "nachrichtenKopf.empfaenger.anschrift", source = "empfaenger.anschrift")
	@Mapping(target = "nachrichtenKopf.absender", source = "absender")
	@Mapping(target = "nachrichtenInhalt.betreff", source = "subject")
	@Mapping(target = "nachrichtenInhalt.storkQaaLevel", source = "storkQaaLevel")
	@Mapping(target = "nachrichtenInhalt.zuVorgang.vorgangsId", source = "vorgangId")
	@Mapping(target = "nachrichtenInhalt.freiText.encoding.schluessel", constant = TEXT_ENCODING_PLAIN_TEXT)
	@Mapping(target = "nachrichtenInhalt.freiText.encoding.tabelle", constant = TABELLE_NUMMER_TEXT_ENCODING)
	@Mapping(target = "nachrichtenInhalt.freiText.text", source = "text")
	@Mapping(target = "nachrichtenInhalt.dataContainer", source = "attachments")
	BspNachricht toBspNachricht(BayernIdMessage message);

	@Mapping(target = "gemeindeschluessel.tabelle", constant = TABELLE_NUMMER_GEMEINDE_SCHLUESSEL)
	@Mapping(target = "gemeindeschluessel.schluessel", source = "gemeindeschluessel")
	AbsenderType toAbsenderType(Absender absender);

	default DataContainerType toDataContainerType(Attachment attachment) {
		var dataContainer = new DataContainerType();
		dataContainer.setFileType(toSchluesseltabelleType(attachment));
		dataContainer.setFileName(attachment.getName());
		try {
			var content = attachment.getContent();
			try {
				dataContainer.setInhalt(content.readAllBytes());
			} finally {
				content.close();
			}
		} catch (IOException e) {
			throw new TechnicalException("Error reading attachment content", e);
		}
		return dataContainer;
	}

	default SchluesseltabelleType toSchluesseltabelleType(Attachment attachment) {
		var schluesseltabelle = new SchluesseltabelleType();
		schluesseltabelle.setSchluessel(attachment.getType());
		schluesseltabelle.setTabelle(TABELLE_NUMMER_MIME_TYPE);
		return schluesseltabelle;
	}

	@Mapping(target = "status", source = "ergebnisStatus.schluessel")
	@Mapping(target = "message", source = "ergaenzendeHinweise")
	@Mapping(target = "success", source = "annahmeErfolgreich")
	BayernIdResponse toBayernIdResponse(BspQuittung quittung);
}
